@@ -1,23 +1,9 @@
+// SPDX-FileCopyrightText: 2011 Lai Jiangshan <laijs@cn.fujitsu.com>
+//
+// SPDX-License-Identifier: LGPL-2.1-or-later
+
 /*
- * rculfhash-mm-mmap.c
- *
  * mmap/reservation based memory management for Lock-Free RCU Hash Table
- *
- * Copyright 2011 - Lai Jiangshan <laijs@cn.fujitsu.com>
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <unistd.h>
@@ -132,8 +118,8 @@ void cds_lfht_alloc_bucket_table(struct cds_lfht *ht, unsigned long order)
 	if (order == 0) {
 		if (ht->min_nr_alloc_buckets == ht->max_nr_buckets) {
 			/* small table */
-			ht->tbl_mmap = calloc(ht->max_nr_buckets,
-					sizeof(*ht->tbl_mmap));
+			ht->tbl_mmap = ht->alloc->calloc(ht->alloc->state,
+					ht->max_nr_buckets, sizeof(*ht->tbl_mmap));
 			urcu_posix_assert(ht->tbl_mmap);
 			return;
 		}
@@ -164,7 +150,7 @@ void cds_lfht_free_bucket_table(struct cds_lfht *ht, unsigned long order)
 	if (order == 0) {
 		if (ht->min_nr_alloc_buckets == ht->max_nr_buckets) {
 			/* small table */
-			poison_free(ht->tbl_mmap);
+			poison_free(ht->alloc, ht->tbl_mmap);
 			return;
 		}
 		/* large table */
@@ -188,7 +174,7 @@ struct cds_lfht_node *bucket_at(struct cds_lfht *ht, unsigned long index)
 
 static
 struct cds_lfht *alloc_cds_lfht(unsigned long min_nr_alloc_buckets,
-		unsigned long max_nr_buckets)
+		unsigned long max_nr_buckets, const struct cds_lfht_alloc *alloc)
 {
 	unsigned long page_bucket_size;
 
@@ -203,7 +189,7 @@ struct cds_lfht *alloc_cds_lfht(unsigned long min_nr_alloc_buckets,
 	}
 
 	return __default_alloc_cds_lfht(
-			&cds_lfht_mm_mmap, sizeof(struct cds_lfht),
+			&cds_lfht_mm_mmap, alloc, sizeof(struct cds_lfht),
 			min_nr_alloc_buckets, max_nr_buckets);
 }
 
