@@ -1,24 +1,10 @@
+// SPDX-FileCopyrightText: 2010 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+// SPDX-FileCopyrightText: 2010 Paolo Bonzini <pbonzini@redhat.com>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 /*
- * test_urcu_wfcq.c
- *
  * Userspace RCU library - example RCU-based lock-free concurrent queue
- *
- * Copyright February 2010 - Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- * Copyright February 2010 - Paolo Bonzini <pbonzini@redhat.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include <stdio.h>
@@ -31,13 +17,12 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <stdio.h>
-#include <assert.h>
 #include <errno.h>
 
 #include <urcu/arch.h>
+#include <urcu/assert.h>
 #include <urcu/tls-compat.h>
 #include <urcu/uatomic.h>
-#include "cpuset.h"
 #include "thread-id.h"
 
 /* hardcoded number of CPUs */
@@ -57,7 +42,7 @@ static enum test_sync test_sync;
 
 static int test_force_sync;
 
-static volatile int test_go, test_stop_enqueue, test_stop_dequeue;
+static volatile int test_stop_enqueue, test_stop_dequeue;
 
 static unsigned long rduration;
 
@@ -75,7 +60,7 @@ static inline void loop_sleep(unsigned long loops)
 static int verbose_mode;
 
 static int test_dequeue, test_splice, test_wait_empty;
-static int test_enqueue_stopped;
+static unsigned int test_enqueue_stopped;
 
 #define printf_verbose(fmt, args...)		\
 	do {					\
@@ -91,7 +76,7 @@ pthread_mutex_t affinity_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static void set_affinity(void)
 {
-#if HAVE_SCHED_SETAFFINITY
+#ifdef HAVE_SCHED_SETAFFINITY
 	cpu_set_t mask;
 	int cpu, ret;
 #endif /* HAVE_SCHED_SETAFFINITY */
@@ -99,7 +84,7 @@ static void set_affinity(void)
 	if (!use_affinity)
 		return;
 
-#if HAVE_SCHED_SETAFFINITY
+#ifdef HAVE_SCHED_SETAFFINITY
 	ret = pthread_mutex_lock(&affinity_mutex);
 	if (ret) {
 		perror("Error in pthread mutex lock");
@@ -114,11 +99,7 @@ static void set_affinity(void)
 
 	CPU_ZERO(&mask);
 	CPU_SET(cpu, &mask);
-#if SCHED_SETAFFINITY_ARGS == 2
-	sched_setaffinity(0, &mask);
-#else
 	sched_setaffinity(0, sizeof(mask), &mask);
-#endif
 #endif /* HAVE_SCHED_SETAFFINITY */
 }
 
@@ -127,12 +108,12 @@ static void set_affinity(void)
  */
 static int test_duration_dequeue(void)
 {
-	return !test_stop_dequeue;
+	return !uatomic_load(&test_stop_dequeue, CMM_RELAXED);
 }
 
 static int test_duration_enqueue(void)
 {
-	return !test_stop_enqueue;
+	return !uatomic_load(&test_stop_enqueue, CMM_RELAXED);
 }
 
 static DEFINE_URCU_TLS(unsigned long long, nr_dequeues);
@@ -160,10 +141,7 @@ static void *thr_enqueuer(void *_count)
 
 	set_affinity();
 
-	while (!test_go)
-	{
-	}
-	cmm_smp_mb();
+	wait_until_go();
 
 	for (;;) {
 		struct cds_wfcq_node *node = malloc(sizeof(*node));
@@ -238,7 +216,7 @@ static void do_test_splice(enum test_sync sync)
 
 	switch (ret) {
 	case CDS_WFCQ_RET_WOULDBLOCK:
-		assert(0);	/* blocking call */
+		urcu_posix_assert(0);	/* blocking call */
 		break;
 	case CDS_WFCQ_RET_DEST_EMPTY:
 		URCU_TLS(nr_splice)++;
@@ -246,7 +224,7 @@ static void do_test_splice(enum test_sync sync)
 		/* ok */
 		break;
 	case CDS_WFCQ_RET_DEST_NON_EMPTY:
-		assert(0);	/* entirely unexpected */
+		urcu_posix_assert(0);	/* entirely unexpected */
 		break;
 	case CDS_WFCQ_RET_SRC_EMPTY:
 		/* ok, we could even skip iteration on dest if we wanted */
@@ -271,10 +249,7 @@ static void *thr_dequeuer(void *_count)
 
 	set_affinity();
 
-	while (!test_go)
-	{
-	}
-	cmm_smp_mb();
+	wait_until_go();
 
 	for (;;) {
 		if (test_dequeue && test_splice) {
@@ -308,8 +283,8 @@ static void *thr_dequeuer(void *_count)
 	return ((void*)2);
 }
 
-static void test_end(unsigned long long *nr_dequeues,
-		unsigned long long *nr_dequeue_last)
+static void test_end(unsigned long long *nr_dequeues_l,
+		unsigned long long *nr_dequeue_last_l)
 {
 	struct cds_wfcq_node *node;
 	int state;
@@ -319,14 +294,14 @@ static void test_end(unsigned long long *nr_dequeues,
 				&state);
 		if (node) {
 			if (state & CDS_WFCQ_STATE_LAST)
-				(*nr_dequeue_last)++;
+				(*nr_dequeue_last_l)++;
 			free(node);
-			(*nr_dequeues)++;
+			(*nr_dequeues_l)++;
 		}
 	} while (node);
 }
 
-static void show_usage(int argc, char **argv)
+static void show_usage(char **argv)
 {
 	printf("Usage : %s nr_dequeuers nr_enqueuers duration (s) <OPTIONS>\n",
 		argv[0]);
@@ -357,27 +332,28 @@ int main(int argc, char **argv)
 			   tot_splice = 0, tot_dequeue_last = 0;
 	unsigned long long end_dequeues = 0;
 	int i, a, retval = 0;
+	unsigned int i_thr;
 
 	if (argc < 4) {
-		show_usage(argc, argv);
+		show_usage(argv);
 		return -1;
 	}
 
 	err = sscanf(argv[1], "%u", &nr_dequeuers);
 	if (err != 1) {
-		show_usage(argc, argv);
+		show_usage(argv);
 		return -1;
 	}
 
 	err = sscanf(argv[2], "%u", &nr_enqueuers);
 	if (err != 1) {
-		show_usage(argc, argv);
+		show_usage(argv);
 		return -1;
 	}
-	
+
 	err = sscanf(argv[3], "%lu", &duration);
 	if (err != 1) {
-		show_usage(argc, argv);
+		show_usage(argv);
 		return -1;
 	}
 
@@ -387,7 +363,7 @@ int main(int argc, char **argv)
 		switch (argv[i][1]) {
 		case 'a':
 			if (argc < i + 2) {
-				show_usage(argc, argv);
+				show_usage(argv);
 				return -1;
 			}
 			a = atoi(argv[++i]);
@@ -397,14 +373,14 @@ int main(int argc, char **argv)
 			break;
 		case 'c':
 			if (argc < i + 2) {
-				show_usage(argc, argv);
+				show_usage(argv);
 				return -1;
 			}
 			rduration = atol(argv[++i]);
 			break;
 		case 'd':
 			if (argc < i + 2) {
-				show_usage(argc, argv);
+				show_usage(argv);
 				return -1;
 			}
 			wdelay = atol(argv[++i]);
@@ -471,24 +447,24 @@ int main(int argc, char **argv)
 
 	next_aff = 0;
 
-	for (i = 0; i < nr_enqueuers; i++) {
-		err = pthread_create(&tid_enqueuer[i], NULL, thr_enqueuer,
-				     &count_enqueuer[3 * i]);
+	for (i_thr = 0; i_thr < nr_enqueuers; i_thr++) {
+		err = pthread_create(&tid_enqueuer[i_thr], NULL, thr_enqueuer,
+				     &count_enqueuer[3 * i_thr]);
 		if (err != 0)
 			exit(1);
 	}
-	for (i = 0; i < nr_dequeuers; i++) {
-		err = pthread_create(&tid_dequeuer[i], NULL, thr_dequeuer,
-				     &count_dequeuer[4 * i]);
+	for (i_thr = 0; i_thr < nr_dequeuers; i_thr++) {
+		err = pthread_create(&tid_dequeuer[i_thr], NULL, thr_dequeuer,
+				     &count_dequeuer[4 * i_thr]);
 		if (err != 0)
 			exit(1);
 	}
 
 	cmm_smp_mb();
 
-	test_go = 1;
+	begin_test();
 
-	for (i = 0; i < duration; i++) {
+	for (i_thr = 0; i_thr < duration; i_thr++) {
 		sleep(1);
 		if (verbose_mode) {
 			fwrite(".", sizeof(char), 1, stdout);
@@ -496,7 +472,7 @@ int main(int argc, char **argv)
 		}
 	}
 
-	test_stop_enqueue = 1;
+	uatomic_store(&test_stop_enqueue, 1, CMM_RELEASE);
 
 	if (test_wait_empty) {
 		while (nr_enqueuers != uatomic_read(&test_enqueue_stopped)) {
@@ -507,26 +483,26 @@ int main(int argc, char **argv)
 		}
 	}
 
-	test_stop_dequeue = 1;
+	uatomic_store(&test_stop_dequeue, 1, CMM_RELAXED);
 
-	for (i = 0; i < nr_enqueuers; i++) {
-		err = pthread_join(tid_enqueuer[i], &tret);
+	for (i_thr = 0; i_thr < nr_enqueuers; i_thr++) {
+		err = pthread_join(tid_enqueuer[i_thr], &tret);
 		if (err != 0)
 			exit(1);
-		tot_enqueues += count_enqueuer[3 * i];
-		tot_successful_enqueues += count_enqueuer[3 * i + 1];
-		tot_empty_dest_enqueues += count_enqueuer[3 * i + 2];
+		tot_enqueues += count_enqueuer[3 * i_thr];
+		tot_successful_enqueues += count_enqueuer[3 * i_thr + 1];
+		tot_empty_dest_enqueues += count_enqueuer[3 * i_thr + 2];
 	}
-	for (i = 0; i < nr_dequeuers; i++) {
-		err = pthread_join(tid_dequeuer[i], &tret);
+	for (i_thr = 0; i_thr < nr_dequeuers; i_thr++) {
+		err = pthread_join(tid_dequeuer[i_thr], &tret);
 		if (err != 0)
 			exit(1);
-		tot_dequeues += count_dequeuer[4 * i];
-		tot_successful_dequeues += count_dequeuer[4 * i + 1];
-		tot_splice += count_dequeuer[4 * i + 2];
-		tot_dequeue_last += count_dequeuer[4 * i + 3];
+		tot_dequeues += count_dequeuer[4 * i_thr];
+		tot_successful_dequeues += count_dequeuer[4 * i_thr + 1];
+		tot_splice += count_dequeuer[4 * i_thr + 2];
+		tot_dequeue_last += count_dequeuer[4 * i_thr + 3];
 	}
-	
+
 	test_end(&end_dequeues, &tot_dequeue_last);
 
 	printf_verbose("total number of enqueues : %llu, dequeues %llu\n",
@@ -579,5 +555,6 @@ int main(int argc, char **argv)
 	free(count_dequeuer);
 	free(tid_enqueuer);
 	free(tid_dequeuer);
+
 	return retval;
 }
